package com.ruoyi.oa.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.oa.mapper.CmcSubContractMapper;
import com.ruoyi.oa.domain.CmcSubContract;
import com.ruoyi.oa.service.ICmcSubContractService;

/**
 * cmc分包合同评审Service业务层处理
 * 
 * @author cmc
 * @date 2024-04-18
 */
@Service
public class CmcSubContractServiceImpl implements ICmcSubContractService 
{
    @Autowired
    private CmcSubContractMapper cmcSubContractMapper;

    /**
     * 查询cmc分包合同评审
     * 
     * @param subContractId cmc分包合同评审主键
     * @return cmc分包合同评审
     */
    @Override
    public CmcSubContract selectCmcSubContractBySubContractId(String subContractId)
    {
        return cmcSubContractMapper.selectCmcSubContractBySubContractId(subContractId);
    }

    /**
     * 查询cmc分包合同评审列表
     * 
     * @param cmcSubContract cmc分包合同评审
     * @return cmc分包合同评审
     */
    @Override
    public List<CmcSubContract> selectCmcSubContractList(CmcSubContract cmcSubContract)
    {
        return cmcSubContractMapper.selectCmcSubContractList(cmcSubContract);
    }

    /**
     * 关联项目查询cmc分包合同评审列表
     *
     * @param cmcSubContract cmc分包合同评审
     * @return cmc分包合同评审
     */
    @Override
    public List<CmcSubContract> selectCmcSubContractListP(CmcSubContract cmcSubContract)
    {
        return cmcSubContractMapper.selectCmcSubContractListP(cmcSubContract);
    }

    /**
     * 按时间范围查询cmc分包合同评审列表
     *
     * @param cmcSubContract cmc分包合同评审
     * @return cmc分包合同评审
     */
    @Override
    public List<CmcSubContract> selectCmcSubContractListByRange(CmcSubContract cmcSubContract)
    {
        return cmcSubContractMapper.selectCmcSubContractListByRange(cmcSubContract);
    }

    /**
     * 新增cmc分包合同评审
     * 
     * @param cmcSubContract cmc分包合同评审
     * @return 结果
     */
    @Override
    public int insertCmcSubContract(CmcSubContract cmcSubContract)
    {
        return cmcSubContractMapper.insertCmcSubContract(cmcSubContract);
    }

    /**
     * 修改cmc分包合同评审
     * 
     * @param cmcSubContract cmc分包合同评审
     * @return 结果
     */
    @Override
    public int updateCmcSubContract(CmcSubContract cmcSubContract)
    {
        return cmcSubContractMapper.updateCmcSubContract(cmcSubContract);
    }

    /**
     * 批量删除cmc分包合同评审
     * 
     * @param subContractIds 需要删除的cmc分包合同评审主键
     * @return 结果
     */
    @Override
    public int deleteCmcSubContractBySubContractIds(String[] subContractIds)
    {
        return cmcSubContractMapper.deleteCmcSubContractBySubContractIds(subContractIds);
    }

    /**
     * 删除cmc分包合同评审信息
     * 
     * @param subContractId cmc分包合同评审主键
     * @return 结果
     */
    @Override
    public int deleteCmcSubContractBySubContractId(String subContractId)
    {
        return cmcSubContractMapper.deleteCmcSubContractBySubContractId(subContractId);
    }
}
